@@ -723,7 +723,8 @@ gtk_application_window_removed (GtkApplication *application,
 
   old_active = priv->windows;
 
-  gtk_application_impl_window_removed (application->priv->impl, window);
+  if (priv->impl)
+    gtk_application_impl_window_removed (priv->impl, window);
 
   g_signal_handlers_disconnect_by_func (window,
                                         gtk_application_focus_in_event_cb,
@@ -733,9 +734,9 @@ gtk_application_window_removed (GtkApplication *application,
   priv->windows = g_list_remove (priv->windows, window);
   gtk_window_set_application (window, NULL);
 
-  if (priv->windows != old_active)
+  if (priv->windows != old_active && priv->impl)
     {
-      gtk_application_impl_active_window_changed (application->priv->impl, priv->windows ? priv->windows->data : NULL);
+      gtk_application_impl_active_window_changed (priv->impl, priv->windows ? priv->windows->data : NULL);
       g_object_notify_by_pspec (G_OBJECT (application), gtk_application_props[PROP_ACTIVE_WINDOW]);
     }
 }
@@ -1019,6 +1020,10 @@ gtk_application_new (const gchar       *application_id,
  *
  * Adds a window to @application.
  *
+ * This call can only happen after the @application has started;
+ * typically, you should add new application windows in response
+ * to the emission of the #GApplication::activate signal.
+ *
  * This call is equivalent to setting the #GtkWindow:application
  * property of @window to @application.
  *
@@ -1026,7 +1031,7 @@ gtk_application_new (const gchar       *application_id,
  * will remain until the window is destroyed, but you can explicitly
  * remove it with gtk_application_remove_window().
  *
- * GTK+ will keep the application running as long as it has
+ * GTK+ will keep the @application running as long as it has
  * any windows.
  *
  * Since: 3.0
@@ -1037,6 +1042,13 @@ gtk_application_add_window (GtkApplication *application,
 {
   g_return_if_fail (GTK_IS_APPLICATION (application));
 
+  if (!g_application_get_is_registered (G_APPLICATION (application)))
+    {
+      g_critical ("New application windows must be added after the "
+                  "GApplication::startup signal has been emitted.");
+      return;
+    }
+
   if (!g_list_find (application->priv->windows, window))
     g_signal_emit (application,
                    gtk_application_signals[WINDOW_ADDED], 0, window);
