@@ -120,52 +120,6 @@ error_unhandled_tag (ParserData   *data,
                data->filename, line, col, tag);
 }
 
-gboolean
-_gtk_builder_boolean_from_string (const gchar  *string,
-                                  gboolean     *value,
-                                  GError      **error)
-{
-  gboolean retval = TRUE;
-  int length;
-
-  g_assert (string != NULL);
-  length = strlen (string);
-
-  if (length == 0)
-    retval = FALSE;
-  else if (length == 1)
-    {
-      gchar c = g_ascii_tolower (string[0]);
-      if (c == 'y' || c == 't' || c == '1')
-        *value = TRUE;
-      else if (c == 'n' || c == 'f' || c == '0')
-        *value = FALSE;
-      else
-        retval = FALSE;
-    }
-  else
-    {
-      gchar *lower = g_ascii_strdown (string, length);
-
-      if (strcmp (lower, "yes") == 0 || strcmp (lower, "true") == 0)
-        *value = TRUE;
-      else if (strcmp (lower, "no") == 0 || strcmp (lower, "false") == 0)
-        *value = FALSE;
-      else
-        retval = FALSE;
-      g_free (lower);
-    }
-
-  if (!retval)
-    g_set_error (error,
-                 GTK_BUILDER_ERROR,
-                 GTK_BUILDER_ERROR_INVALID_VALUE,
-                 "Could not parse boolean '%s'",
-                 string);
-
-  return retval;
-}
-
 static GObject *
 builder_construct (ParserData  *data,
                    ObjectInfo  *object_info,
@@ -681,9 +635,8 @@ parse_property (ParserData   *data,
 static void
 free_property_info (PropertyInfo *info)
 {
-  g_free (info->data);
+  g_string_free (info->text, TRUE);
   g_free (info->context);
-  /* info->text is already freed */
   g_slice_free (PropertyInfo, info);
 }
 
@@ -973,7 +926,7 @@ start_element (GMarkupParseContext  *context,
   ParserData *data = (ParserData*)user_data;
 
 #ifdef G_ENABLE_DEBUG
-  if (gtk_get_debug_flags () & GTK_DEBUG_BUILDER)
+  if (GTK_DEBUG_CHECK (BUILDER))
     {
       GString *tags = g_string_new ("");
       int i;
@@ -1157,8 +1110,6 @@ end_element (GMarkupParseContext  *context,
               g_string_assign (prop_info->text, translated);
             }
 
-          prop_info->data = g_string_free (prop_info->text, FALSE);
-
           object_info->properties = g_slist_prepend (object_info->properties, prop_info);
         }
       else
